<?php

namespace Teknisa\Libs\Service;

use Teknisa\Libs\Exception\Login as LoginException;
use Teknisa\Libs\Util\Utilities;

class Menu {
    const ALL_ELEMENTS_VISIBLE = 'ALL';
    const ZHU_LOG_PACKAGE = 'zhulog';
    const ZHU_LOG_CONTAINER = 'log';

    protected $menuConfigPath;

    public function __construct() {
        $config = json_decode(file_get_contents(__DIR__ . '/../../../../config/config.json'), true);
        $this->menuConfigPath = $config['menuConfigPath'];
    }

    /**
     * @param bool $concatenateGroupMenu
     * @param array $visibleMenus
     * @param array $urlOrigin
     * @param array $currentModule
     * @return array
     * @throws \Exception
     */
    public function buildMenu($concatenateGroupMenu, $visibleMenus, $urlOrigin, $currentModule) {
        $menuConfig     = $this->getMenuConfig();
        $modulesConfig  = $this->getJsonMetadata($menuConfig['modulesFile'], true);
        $routesAndMenus = $this->getMenusMetadata($menuConfig, $modulesConfig);
        $menus          = $this->joinMenus($routesAndMenus['MENUS']);
        $menus          = $this->mergeMenus($menus, $urlOrigin, $modulesConfig, $currentModule);

        $this->removeInvisibleMenus($menus, $visibleMenus);

        if($concatenateGroupMenu) {
            $menus = $this->concatenateGroupMenu($menuConfig, $menus, $modulesConfig);
        }

        return array('MENUS' => $menus,
            'ROUTES_TRANSLATE' => $this->generateRoutesTranslateDataSource($routesAndMenus['ROUTES_TRANSLATE'])
        );
    }

    /**
     * @return array
     * @throws \Exception
     */
    private function getMenuConfig()
    {
        $menuConfig = @file_get_contents($this->menuConfigPath);
        if (!$menuConfig) {
            LoginException::menuConfigNotFound();
        }
        return json_decode($menuConfig, true);
    }

    /**
     * @param $menuConfig
     * @return array
     * @throws \Exception
     */
    private function getContainersJson($menuConfig)
    {
        $containers = @file_get_contents($this->updateFilePath($menuConfig['containersFile']));
        if (!$containers) {
            LoginException::containersNotFound();
        }
        return json_decode($containers, true);
    }

    /**
     * @param $modulesFilePath
     * @param bool $isException
     * @throws \Exception
     * @return array
     */
    private function getJsonMetadata($modulesFilePath, $isException = false)
    {
        $modulesConfig = @file_get_contents($this->updateFilePath($modulesFilePath));
        if (!$modulesConfig && $isException) {
            LoginException::modulesConfigNotFound();
        }
        return $modulesConfig ? json_decode($modulesConfig, true) : $modulesConfig;
    }

    private function addDevOpsIdInMenus(&$menus, $devOpsId) {        
        foreach($menus as $key => &$menu) {
            $menu['devOpsId'] = $devOpsId;
            if(isset($menu['menus'])) {
                $this->addDevOpsIdInMenus($menu['menus'], $devOpsId);
            }
        }
    }

    /**
     * @param $menuConfig
     * @param $modulesConfig
     * @return array
     */
    private function getMenusMetadata($menuConfig, $modulesConfig)
    {
        $menus = $routesTranslate = array();
        $keyModulesConfig = array_keys($modulesConfig);
        $productMenu = $this->getJsonMetadata($menuConfig['parentAclStructFile']);
        if ($productMenu) {
            $this->addDevOpsIdInMenus($productMenu, $modulesConfig['product']['devOpsId']);
            $menus[] = $productMenu;
        }
        $modulesMenuPaths = $menuConfig['modulesMenuFile'];
        if(isset($menuConfig['parentMenuConfig'])) {
            $parentMenuConfig = $this->getJsonMetadata($menuConfig['parentMenuConfig']);
            if ($parentMenuConfig) {
                $modulesMenuPaths = $parentMenuConfig['modulesMenuFile'];
            }
        }
        foreach ($modulesMenuPaths as $moduleName => $modulePath) {
            $this->moduleName = $moduleName;
            if (in_array($moduleName, $keyModulesConfig)) {
                $moduleMenu = $this->getJsonMetadata($modulePath);
                if ($moduleMenu) {
                    $this->addDevOpsIdInMenus($moduleMenu, $modulesConfig[$moduleName]['devOpsId']);
                    $menus[] = $moduleMenu;
                }
                $routeTranslate = $this->getRouteTranslate($menuConfig, $modulePath, $moduleName);
                if (!empty($routeTranslate)) {
                    $routesTranslate = array_merge($routesTranslate, $routeTranslate);
                }
            }
        }

        return array('MENUS' => $menus, 'ROUTES_TRANSLATE' => $routesTranslate);
    }

    /**
     * @param $menus
     * @return array
     */
    private function joinMenus($menus)
    {
        $menusAux = array();
        foreach ($menus as $menuGrouped) {
            $menusAux = array_merge($menusAux, $menuGrouped);
        }
        return $menusAux;
    }

    /**
     * @param $menus
     * @param $filterCache
     * @param $urlOrigin
     * @param $modulesConfig
     * @param $currentModule
     * @return array
     */
    private function filterMenus($menus, $filterCache, $urlOrigin, $modulesConfig, $currentModule) {
        $menusAux = [];
        foreach($menus as $menu) {
            $show = true;
            $menuName = isset($menu['name']) ? $menu['name'] : $menu['label'];
            if(isset($filterCache[$menuName])) {
                $show = false;
                $cachedMenu = $filterCache[$menuName];
                if(isset($cachedMenu['menus'])) {
                    $cachedMenu['menus'] = $this->overrideMenus($cachedMenu, $menu);
                    foreach($menusAux as &$menuAux) {
                        $menuAuxName = isset($menuAux['name']) ? $menuAux['name'] : $menuAux['label'];
                        $cachedMenuName = isset($cachedMenu['name']) ? $cachedMenu['name'] : $cachedMenu['label'];
                        if($menuAuxName == $cachedMenuName) {
                            $menuAux['menus'] = array_merge($menuAux['menus'], $cachedMenu['menus']);
                        }
                    }
                }
            } else {
                $cachedMenu = $filterCache[$menuName] = $menu;
            }

            if(isset($cachedMenu['menus'])) {
                foreach($cachedMenu['menus'] as $childrenMenu) {
                    if(isset($childrenMenu['menus'])) {
                        $cache = [];
                        $childrenMenu['menus'] = $this->filterMenus($childrenMenu['menus'], $cache, $urlOrigin,
                            $modulesConfig, $currentModule);
                    }
                }
            }

            if($urlOrigin) {
                $menu = $this->updateMenuURL($menu, $urlOrigin, $modulesConfig, $currentModule);
            }
            if($show) {
                $menusAux[] = $menu;
            }
        }

        return $menusAux;
    }

    private function updateMenuURL($menu, $urlOrigin, $modulesConfig, $currentModule) {
        if(isset($menu['windowName'])) {
            $window = explode('#', $menu['windowName']);
            if(count($window) == 1) {
                $module    = $currentModule;
                $container = $window[0];
            } else {
                $module    = explode('_', $window[0])[0];
                $container = $window[1];                
            }
            if($module != $currentModule && $module != self::ZHU_LOG_PACKAGE && $container != self::ZHU_LOG_CONTAINER) {
                $menu['url'] = $urlOrigin . '/' . $modulesConfig[$module]['baseUrl'] . "/#/" . $menu['windowName'];
                $menu['msContainerName'] = $menu['windowName'];
                $menu['events'] = array(
                    array(
                        'name' => 'MenuOnClick',
                        'code' => 'LibUtilities.openMicroServiceUrl(args)'
                    )
                );
                unset($menu['windowName']);
            }
        }

        return $menu;
    }

    /**
     * @param $uncheckedMenus
     * @param $urlOrigin
     * @param $modulesConfig
     * @param $currentModule
     * @return array
     */
    private function mergeMenus($uncheckedMenus, $urlOrigin, $modulesConfig, $currentModule) {
        $filterCache  = [];
        $checkedMenus = $this->filterMenus($uncheckedMenus, $filterCache, $urlOrigin, $modulesConfig, $currentModule);
        foreach($checkedMenus as &$checkedMenu) {
            if(isset($checkedMenu['menus'])) {
                $checkedMenu['menus'] = $this->mergeMenus($checkedMenu['menus'], $urlOrigin, $modulesConfig, $currentModule);
            }
        }

        return $checkedMenus;
    }

    /**
     * @param $cachedMenu
     * @param $menu
     * @return array
     */
    private function overrideMenus($cachedMenu, $menu)
    {
        $cachedMenu['menus'] = array_merge($cachedMenu['menus'], $menu['menus']);
        $menusToOverride = array_filter($cachedMenu['menus'], function ($menu) {
            return isset($menu['override']);
        });
        $cachedMenusAux = [];
        foreach ($cachedMenu['menus'] as $cached) {
            $shouldFilter = false;
            foreach ($menusToOverride as &$menuToOverride) {
                $overrideMenuId = isset($menuToOverride['id']) ? $menuToOverride['id'] : null;
                $cachedMenuId = isset($cached['id']) ? $cached['id'] : false;
                $shouldFilter = $cachedMenuId        !== $overrideMenuId &&
                    ((isset($cached['windowName']) && $cached['windowName'] == $menuToOverride['override']) ||
                     $cached['name'] == $menuToOverride['override']);
                if ($shouldFilter && isset($cached['menus']) && isset($menuToOverride['menus'])) {
                    $menuToOverride['menus'] = array_merge($menuToOverride['menus'], $cached['menus']);
                }
                if ($shouldFilter) {
                    break;
                }
            }
            if (!$shouldFilter) {
                $cachedMenusAux[] = $cached;
            }
        }
        return $cachedMenusAux;
    }

    /**
     * @param $menus
     * @param $visibleMenus
     */
    private function removeInvisibleMenus(&$menus, $visibleMenus) {
        $allMenus = !empty($visibleMenus) && $visibleMenus[0] == self::ALL_ELEMENTS_VISIBLE;
        foreach($menus as $key => &$menu) {
            if ($this->isInvisibleMenu($visibleMenus, $menu, $allMenus)) {
                unset($menus[$key]);
            } else {
                if(isset($menu['menus'])) {
                    $this->removeInvisibleMenus($menu['menus'], $visibleMenus);
                }
            }
        }
        $menus = array_values($menus);
    }

    /**
     * @param $menuConfig
     * @param $aclStruct
     * @return array
     */
    private function concatenateGroupMenu($menuConfig, $aclStruct, $modulesConfig)
    {
        $containers = $this->getContainersJson($menuConfig);
        $menus = [];
        foreach ($containers as $project) {
            if (isset($project['groupMenu'])) {
                foreach ($project['groupMenu'] as &$groupMenu) {
                    $groupMenu['devOpsId'] = $modulesConfig['product']['devOpsId'];
                    foreach ($aclStruct as $struct) {
                        if (isset($struct['parent']) && $struct['parent'] == $groupMenu['name']) {
                            $groupMenu['menus'][] = $struct;
                        }
                    }
                }
                $menus = $project['groupMenu'];
            }
        }
        return $menus;
    }

    /**
     * @param $visibleMenus
     * @param $menu
     * @param $allMenus
     * @return bool
     */
    private function isInvisibleMenu($visibleMenus, $menu, $allMenus)
    {
        return (!$allMenus &&
                (
                 (isset($menu['isVisible']) && $menu['isVisible'] == false) ||
                 (!isset($menu['id'])) ||
                 ($visibleMenus !== null && !in_array($menu['id'], $visibleMenus))
                )
               );
    }

    private function updateFilePath($path) {
        return Utilities::getProductBasePath() . $path;
    }

    /**
     * @param $menuConfig
     * @param $modulePath
     * @param $moduleName
     * @return array
     */
    private function getRouteTranslate($menuConfig, $modulePath, $moduleName)
    {
        $modulePathArr = explode('/', $modulePath);
        if ($moduleName == Utilities::PRODUCT_MENU_NAME) {
            $modulePathArr = explode('/', $menuConfig['parentAclStructFile']);
        }
        array_pop($modulePathArr);
        $routeTranslateFile = join('/', $modulePathArr) . '/routeTranslate.json';
        return $this->getJsonMetadata($routeTranslateFile);
    }

    private function generateRoutesTranslateDataSource($routesTranslate) {
        array_walk($routesTranslate, function(&$routeLabel, $route) {
            $routeLabel = array(
                'ROUTE' => $route,
                'ROUTE_LABEL' => $routeLabel
            );
        });

        return array_values($routesTranslate);
    }
} 